%==========================================================================
% Semantic Flow for Complex-Event Recognition in Videos
% Configurations for scenario parsing
% 2010. 1. 5. Suha Kwak, POSTECH.
%==========================================================================



function config



%% DEFINE NOTATIONS
% capital characters for symbolic representations of each element
char_idx = 64;

% pre-defined symbols for relationships among elements
temp_nonadj = '<';      % A<B : end(A) < start(B)
during      = '^';      % A^B : start(A) < start(B) and end(B) < end(A)
start_ahead = '~';      % A~B : start(A) < start(B)
no_ordering = '&';      % A&B : no temporal relationships (logical AND)
selection   = '|';      % A|B : only one of A and B can appear (logical OR)
iter_occur  = '*';      % A*  : unknown number of iterative occurrences

% pre-defined symbol to represent empty elements
dummy       = '#';      % A|# : A or not, a dummy element should be only used with OR operator

% integer values for ancestors (0 : no ancestors)
stt_TN = 1;             % starting time ancestor for "temporal non-adjacency"
stt_DU = 2;             % starting time ancestor for "during"
stt_SA = 3;             % starting time ancestor for "start ahead"
end_DU = 4;             % ending time ancestor for "during"

% possible conditions for each element
cond_num = 3;           % except 'cond_E' and 'cond_I'
cond_E = 0;             % 'excluded' (for elements not selected among a group of OR operator and dummy elements)
% properties of 'cond_E'
    % prop 1. 
        % this condition does not allow any condition-shift from itself
    % prop 2. 
        % elements whose ancestors are on 'cond_E' consider the condition as 'cond_F'
    % note that (X ^ A) is impossible logically where X is a dummy element
cond_R = 1;             % 'ready'
cond_A = 2;             % 'active now'
cond_F = 3;             % 'finished'
cond_I = 4;             % 'iterative-ready' (to recognize iterative occurrences)
% properties of 'cond_I'
    % prop 1.
        % in the view point of outside, 'cond_I' is same with 'cond_A'
        % in the view point of inside, it is same with 'cond_F'
        % ex. A^(B^C)*
            % when C is on 'cond_I', B can shift its condition from 'cond_A' to 'cond_I' (cannot shift to 'cond_F')
            % when B is on 'cond_F', A can shift its condition from 'cond_A' to 'cond_F'
            % when B is on 'cond_I', A cannot shift its condition from
            % 'cond_A' to 'cond_F'
	% prop 2.
        % to deal with nested star closer, there can be several 'cond_I' conditions
        % if there are 5 '*' operators,
            % 'cond_I' for the range of the first '*' = cond_I + 1
            % 'cond_I' for the range of the second '*' = cond_I + 2
            %  ...
            % 'cond_I' for the range of the Nth '*' = cond_I + N (where N
            % is an arbitrary positive integer)
            


%% SAVE THE CONFIGURATIONS
save('config.mat', 'char_idx', ...
                   'temp_nonadj', 'during', 'start_ahead', 'no_ordering', 'selection', 'iter_occur', ...
                   'dummy', ...
                   'stt_TN', 'stt_DU', 'stt_SA', 'end_DU', ...
                   'cond_num', 'cond_E', 'cond_R', 'cond_A', 'cond_F', 'cond_I');
