%==========================================================================
% Constraint Flow for Composite-Event Recognition
% Search the temporally first primitives of the input scenario
% 2011. 9. 29. Suha Kwak, POSTECH.
%==========================================================================



function first_prims = firstPrim(scenario)
	% scenario    : Given scenario
    % first_prims : Output temporally first primitives of the scenario.



%% PRELIMINARY
% pre-defined symbols for relationships among elements
load config

% ignore the unary operator 
% - it does not matter when considering just temporal orders
scenario = scenario(~(scenario == iter_occur));

% primitivies in the scenario
prim_pos = bwconncomp(scenario > 47 & scenario < 58);
prim_num = length(prim_pos.PixelIdxList);
prim_list = cellfun(@(x) uint16(str2double(scenario(x))), prim_pos.PixelIdxList);





%% SEARCH FOR THE TEMPORALLY FIRST PRIMITIVES
% construction of the relative-order matrix ===============================
% a matrix to represent relative temporal order among the primitives
% - nonzero elements in a row indicate the primitives which are activated
%   before the primitive associated with the row index.
order_mat = false(prim_num);

% analyses for all binary temporal relationships --------------------------
oper_pos = find(scenario == temp_nonadj | scenario == during | scenario == start_ahead);
for oper_idx = 1 : length(oper_pos)
	% forward scope of the operator
    forward_scope = foreEffPart(scenario(1 : (oper_pos(oper_idx) - 1)));
    fprim_pos = bwconncomp(forward_scope > 47 & forward_scope < 58);
    forward_lprims = cellfun(@(x) uint16(str2double(forward_scope(x))), fprim_pos.PixelIdxList);
    
    % backward scope of the operator
    backward_scope = hindEffPart(scenario((oper_pos(oper_idx) + 1) : end));
    bprim_pos = bwconncomp(backward_scope > 47 & backward_scope < 58);
    backward_lprims = cellfun(@(x) uint16(str2double(backward_scope(x))), bprim_pos.PixelIdxList);
    
    % 'forward_lprims' are activated before 'backward_lprims'.
    ridx = false(1, prim_num);
    for pidx = 1 : length(backward_lprims)
        ridx = ridx | (prim_list == backward_lprims(pidx));
    end
    cidx = false(1, prim_num);
    for pidx = 1 : length(forward_lprims)
        cidx = cidx | (prim_list == forward_lprims(pidx));
    end
    order_mat(ridx, cidx) = true;
end


% search for the temporally first primitives ==============================
first_prim_list = (sum(order_mat, 2) == 0);
first_prims = prim_list(first_prim_list);


