%==========================================================================
% Semantic Flow for Complex-Event Recognition in Videos
% Segment a fore effective-part of symbolic string
% 2009. 12. 6. Suha Kwak, POSTECH.
%==========================================================================



function fore_effpart = foreEffPart(strpart)
% properties of a fore effective-part for an operator
	% 1. grouped by '&' and '|'
	% 2. grouped by a pair of parentheses
        % ex. A&(B<C)&D<E, 
            % for '<' before 'C', input strpart = 'A&(B'      and fore_effpart = 'B'
            % for '<' before 'E', input strpart = 'A&(B<C)&D' and fore_effpart = 'A&(B<C)&D'
        % ex. A~B^C<D
            % for '<' before 'D', input strpart = 'A~B^C'     and fore_effpart = 'C'

            

%% PRELIMINARY
load config



%% EFFECTIVE PART SEARCH
str_idx = length(strpart);
while str_idx > 0
    nest_paren_num = 0;
    
% search for a parenthesis pair -------------------------------------------
    if strpart(str_idx) == ')'
        % grouped by parentheses
        nest_paren_num = nest_paren_num + 1;
        while nest_paren_num > 0
            str_idx = str_idx - 1;
            if str_idx < 1
                disp('foreEffPart: wrong input string - parenthesis matching');
                break;
            elseif strpart(str_idx) == ')'
                nest_paren_num = nest_paren_num + 1;
            elseif strpart(str_idx) == '('
                nest_paren_num = nest_paren_num - 1;
            else
                % skip (contents in the parentheses is not important)
            end
        end
        
% check around primitives -------------------------------------------------
    elseif strpart(str_idx) > 47 && strpart(str_idx) < 58
        % find the first digit of the primitive
        while strpart(str_idx - 1) > 47 && strpart(str_idx - 1) < 58
            str_idx = str_idx - 1;
            if str_idx == 1
                break;
            end
        end
        % grouped by '&' and '|': search more
        if str_idx > 1 && (strpart(str_idx - 1) == no_ordering || strpart(str_idx - 1) == selection)
            str_idx = str_idx - 1;
        % end of the search
        else
            head_pos = str_idx;
            break;
        end
        
% skip logical relations and unary temporal relations ---------------------
    elseif strpart(str_idx) == no_ordering || strpart(str_idx) == selection
        % grouping by '&' and '|' which are in front of a parenthesis
        
    elseif strpart(str_idx) == iter_occur
        % just skip the operator '*'

% stop when a binary temporal relation or '(' appears ---------------------
    else
        break;
    end
    head_pos = str_idx;
    
    str_idx = str_idx - 1;
end

fore_effpart = strpart(head_pos : length(strpart));
