%==========================================================================
% Semantic Flow for Complex-Event Recognition in Videos
% Generate initial flow nodes by considering OR grouping results
% 2009. 12. 30. Suha Kwak, POSTECH.
%==========================================================================



function init_FN = genInitFNodes(dummy_elems, or_group_map)
    % dummy_elems  : a list of dummy elements
    % or_group_map : a map for OR grouping results



%% PRELIMINARY
% load configuration
load config

% the number of elements
elem_num = length(dummy_elems);

% the number of OR groups
or_group_num = size(or_group_map, 1);



%% INITIAL FLOW NODE GENERATION
% initial flow node without any OR group ==================================
init_FN = uint16(ones(1, elem_num)) .* cond_R;
init_FN(dummy_elems) = cond_E;

% apply OR grouping =======================================================
if or_group_num > 0
    base_init_FN = init_FN;
    init_FN = uint16(false(0));
    
% apply OR grouping result ------------------------------------------------
    % the number of parts of each OR group
    part_num = max(or_group_map, [], 2);

    % combination of parts of each OR group
    part_comb = uint8(false(1, or_group_num));
    for grp_idx = 1 : or_group_num
        comb_num = size(part_comb, 1);
        base_part_comb = part_comb;
        part_comb = uint8(false(0));
        for part_idx = 1 : part_num(grp_idx)
            for cmb_idx = 1 : comb_num
                cur_part_comb = base_part_comb(cmb_idx, :);
                cur_part_comb(grp_idx) = part_idx;
                part_comb = [part_comb ; cur_part_comb];
            end
        end
    end
    
    % generate initial flow nodes from part-combination vectors
    for cmb_idx = 1 : size(part_comb, 1)
        cur_init_FN = base_init_FN;
        part_comb_vec = part_comb(cmb_idx, :);
        excluded_elems = false(1, elem_num);
        for grp_idx = 1 : or_group_num
            excluded_elems = excluded_elems | ((or_group_map(grp_idx, :) ~= part_comb_vec(grp_idx)) & (or_group_map(grp_idx, :) > 0));
        end
        cur_init_FN(excluded_elems) = cond_E;
        init_FN = [init_FN ; cur_init_FN];
    end
    
% remove duplicated initial flow nodes ------------------------------------
    % the number of initial flow nodes
    init_FN_num = size(init_FN, 1);
    
    % a list of valid flow nodes
    valid_FN_list = true(init_FN_num, 1);
    
    % remove duplicated flow nodes
    for init_fn_idx1 = 1 : init_FN_num - 1
        val_FN = init_FN(init_fn_idx1, :);
        for init_fn_idx2 = init_fn_idx1 + 1 : init_FN_num
            if sum(val_FN == init_FN(init_fn_idx2, :)) == elem_num
                valid_FN_list(init_fn_idx2) = false;
            end
        end
    end
    
    % remove invalid flow nodes
    init_FN = init_FN(valid_FN_list, :);
end
